﻿using Microsoft.WindowsAzure.Storage.Blob;
using PetkaWebDav.Server.Framework.BaseClasses;
using PetkaWebDav.Server.Framework.Enums;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PetkaWebDav.Handler.AzureBlob.Handlers
{
    /// <summary>
    /// Custom implementation example for DavPut.
    /// </summary>
    public sealed class DavPut : DavPutBase
    {
        public DavPut()
        {
            this.ProcessDavRequest += new DavProcessEventHandler(DavPut_ProcessDavRequest);
        }

        private void DavPut_ProcessDavRequest(object sender, EventArgs e)
        {
            CloudBlockBlob _blob = RequestWrapper.DownloadBlobAttributes(base.RelativeRequestPath);

            if (_blob == null)
                base.AbortRequest(ServerResponseCode.NotFound);
            else
            {
                if (!base.OverwriteExistingResource)
                {
                    //Check to see if the resource already exists
                    if (_blob != null)
                        base.AbortRequest(DavPutResponseCode.Conflict);
                    else
                        SaveFile(_blob);
                }
                else
                    SaveFile(_blob);
            }
        }

        private void SaveFile(CloudBlockBlob _blob)
        {
            byte[] _requestInput = base.GetRequestInput();

            using (MemoryStream stream = new MemoryStream())
            {
                stream.Write(_requestInput, 0, _requestInput.Length);
                stream.Position = 0;
                RequestWrapper.UploadBlob(_blob.Name, stream);
            }
        }
    }
}
